#include "particle.lua"
#include "generic.lua"
#include "light_spawner/lightspawner.lua"

GroundedEngine_ToggleLightKey = 'L' -- Toggle lights, to give ya a choice
GroundedEngine_EngineLight = nil
GroundedEngine_EmitLight = true

function init()
	joints = FindJoints("engine")
	for j, joint in pairs(joints) do
		SetJointMotor(joint, 20)
	end
	bodies = FindBodies() --gets all bodies under the script because those are all we need (the propellers)
	enginepos = GetLocationTransform(FindLocation("emit")).pos
	enginedamagedpos = GetLocationTransform(FindLocation("emit-damaged")).pos
	enginebreak = 0
	enginebreakShape=FindShape("explosive")
	localEnginepos = TransformToLocalPoint(GetBodyTransform(FindBody("boeing",true)),enginepos)
	localDamagedEnginepos = TransformToLocalPoint(GetBodyTransform(FindBody("boeing",true)),enginedamagedpos)
	flame = 100
	SetInt("level.engines",2)

	GroundedEngine_EngineLight = LightSpawner_Spawn(localEnginepos, 100, 100, {0.93,0.25,0}, false, "engine_light")

	enginebroken = false
	smallsound = {}
	for i=0,5 do
		smallsound[i] = LoadSound("MOD/main/Boeing 737 FUEL/snd/fail_small/"..i)
	end
	bigsound = {}
	for i=0,3 do
		bigsound[i] = LoadSound("MOD/main/Boeing 737 FUEL/snd/fail_big/"..i)
	end
	engineLoop = LoadLoop("MOD/main/Boeing 737 FUEL/snd/engine_exterior.ogg")
end

function tick(dt)
	local last_pressed = InputLastPressedKey()
	if last_pressed == GroundedEngine_ToggleLightKey then
		if GroundedEngine_EmitLight then
			GroundedEngine_EmitLight = false
			LightSpawner_UpdateLightEnabled(GroundedEngine_EngineLight, false)
		else
			GroundedEngine_EmitLight = true
		end
	end

	enginepos = TransformToParentPoint(GetBodyTransform(FindBody("boeing",true)),localEnginepos)
	enginedamagedpos = TransformToParentPoint(GetBodyTransform(FindBody("boeing",true)),localDamagedEnginepos)
	-- enginerot = TransformToParentPoint(GetBodyTransform(FindBody("boeing",true)), localEnginerot)

	local localbehind = Vec(0.5, 0, 60)
	local enginerot = TransformToParentVec(GetBodyTransform(FindBody("boeing",true)), localbehind)

	local id = GetIntParam("id")
	red = GetInt("level.engines."..id..".red")

	local ptrans = GetPlayerTransform()
	for b, body in pairs(bodies) do
		center = TransformToParentPoint(GetBodyTransform(body),GetBodyCenterOfMass(body))
		voxels, broken, total = GetBodyVoxelCount(body)
		local soundpos = VecLerp(ptrans.pos,center,0.1)
		--DebugPrint(FindShape("explosive"))
		if voxels == 0 or FindShape("explosive")==0 then --completely destroyed = no vfx
			enginebreak = 2
			flame = -150
			if enginebroken == false then
				PlaySound(bigsound[math.random(1,3)],enginepos,100)
				SetInt("level.engines",GetInt("level.engines")-1)
				enginebroken = true
			end
		end
		if broken > 40 and enginebreak == 0 then
			enginebreak = 1
			PlaySound(smallsound[math.random(1,5)],soundpos,5)
		elseif broken > 60 and enginebreak <= 1 then
			enginebreak = 2
			PlaySound(bigsound[math.random(1,3)],soundpos,100)
			if enginebroken == false then
				SetInt("level.engines",GetInt("level.engines")-1)
				enginebroken = true
			end
		end

	end
	if enginebreak == 2 then --very, very bad things are happening
		-- original code
		-- --first spawn fire
		-- ParticleReset()
		-- ParticleColor(1,0.6,0.3,0.1,0.1,0.1,"linear",0,0.5)
		-- ParticleEmissive(2,0,"linear",0,0.5)
		-- ParticleRadius(3+(flame/400),5,"linear",0.2,1)
		-- ParticleAlpha(1.0,0.0)
		-- ParticleCollide(0)
		-- ParticleGravity(4,9)
		-- SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,4),(flame/50)+1.5)
		-- --next is smoke
		-- ParticleReset()
		-- ParticleColor(0.16,0.15,0.15)
		-- ParticleRadius(3,5)
		-- ParticleAlpha(0.8,0.0,"linear",0.03,0.9)
		-- ParticleCollide(0)
		-- ParticleGravity(4,9)
		-- SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,4),(flame/20)+3.5)

		local fire_particle = Generic_deepCopy(Particle_Fire_Expl)
		local smoke_particle = Generic_deepCopy(Particle_Smoke_Expl)
		for i=0, 5 do
			fire_particle["custom_direction"] = Vec(math.random()-1,math.random()-1,4)
			smoke_particle["custom_direction"] = Vec(math.random()-1,math.random()-1,4)
			smoke_particle["location_random"] = 3
			fire_particle["location_random"] = 3
			Particle_FireSmoke(fire_particle, smoke_particle, 90, 100, VecSub(enginedamagedpos, {0,0.5,0}))
		end

		if GroundedEngine_EmitLight then
			LightSpawner_SetNewLightLocation(GroundedEngine_EngineLight,VecSub(enginedamagedpos, {0,0.5,0}))
			LightSpawner_SetNewLightSize(GroundedEngine_EngineLight, 500)
			LightSpawner_UpdateLightIntensity(GroundedEngine_EngineLight, 500)
			LightSpawner_UpdateLightEnabled(GroundedEngine_EngineLight, true)
		end

		enginebreak = 3
	end

	if enginebreak == 3 then --very, very bad things are happening
		-- original code
		-- --first spawn fire
		-- ParticleReset()
		-- ParticleColor(1,0.6,0.3,0.1,0.1,0.1,"linear",0,0.5)
		-- ParticleEmissive(2,0,"linear",0,0.5)
		-- ParticleRadius(3+(flame/400),5,"linear",0.2,1)
		-- ParticleAlpha(1.0,0.0)
		-- ParticleCollide(0)
		-- ParticleGravity(4,9)
		-- SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,4),(flame/50)+1.5)
		-- --next is smoke
		-- ParticleReset()
		-- ParticleColor(0.16,0.15,0.15)
		-- ParticleRadius(3,5)
		-- ParticleAlpha(0.8,0.0,"linear",0.03,0.9)
		-- ParticleCollide(0)
		-- ParticleGravity(4,9)
		-- SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,4),(flame/20)+3.5)

		local fire_particle = Generic_deepCopy(Particle_Fire)
		local smoke_particle = Generic_deepCopy(Particle_Smoke)
		fire_particle["custom_direction"] = Vec(math.random()-1,math.random()-1,4)
		smoke_particle["custom_direction"] = Vec(math.random()-1,math.random()-1,4)
		Particle_FireSmoke(fire_particle, smoke_particle, 90, 100, VecSub(enginedamagedpos, {0,0.5,0}))


		if GroundedEngine_EmitLight then
			LightSpawner_SetNewLightLocation(GroundedEngine_EngineLight,VecSub(enginedamagedpos, {0,0.5,0}))
			LightSpawner_UpdateLightIntensity(GroundedEngine_EngineLight, Generic_rnd(50, 100))
			LightSpawner_UpdateLightEnabled(GroundedEngine_EngineLight, true)
		end

	end

	if red > 0 and enginebreak < 2 then --funny tomato juice in the engines
		SetInt("level.engines."..id..".red",red-1)

		ParticleReset()
		ParticleColor(0.4,0.2,0.2)
		ParticleRadius(2.1,4)
		ParticleAlpha(0.2,0.0,"linear",0.03,0.9)
		ParticleCollide(0)
		ParticleGravity(0,-2)
		SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,2),7)
	end

	if enginebreak < 2 then
		sceneBodies = FindBodies(nil,true)
		for b=1,#sceneBodies do
			bTrans = GetBodyTransform(sceneBodies[b])
			if VecLength(VecMult(VecSub(bTrans.pos,enginepos),Vec(1,1,0.4))) < 5 then
				ApplyBodyImpulse(sceneBodies[b],bTrans.pos,Vec(0,0,GetBodyMass(sceneBodies[b])))
				--DrawBodyOutline(sceneBodies[b])
			end
		end
		pTrans = GetPlayerTransform()
		if VecLength(VecMult(VecSub(pTrans.pos,enginepos),Vec(1,1,0.4))) < 5 then
			SetPlayerVelocity(VecAdd(GetPlayerVelocity(),Vec(0,0,1)))
			if VecLength(VecSub(pTrans.pos,enginepos))<2.5 then
				SetPlayerHealth(0)
				SetPlayerVelocity(Vec(0,0,5))
				pTrans.pos[3]= pTrans.pos[3]+3
				SetPlayerTransform(pTrans)
			end
		end


		local smoke_particle = Generic_deepCopy(Particle_Smoke)
		smoke_particle["color"] = {r=0.9,g=0.9,b=0.9,a=0.5}
		smoke_particle["lifetime"] = 4
		smoke_particle["drag"] = 0.5
		smoke_particle["gravity"] = 0
		smoke_particle["location_random"] = 3
		for i=0, 2 do
			smoke_particle["custom_direction"] = enginerot
			Particle_EmitParticle(smoke_particle, enginepos, "smoke", 100)
		end
		--DebugPrint(engineLoop)
		PlayLoop(engineLoop,enginepos,2/VecLength(VecSub(pTrans.pos,enginepos)))
	end
end

function GetBodyVoxelCount(body)
	local broken = 0
	local voxels = 0
	local shapes = GetBodyShapes(body)
	for s, shape in pairs(shapes) do
		voxels = voxels + GetShapeVoxelCount(shape)
		if IsShapeBroken(shape) then
			broken = broken + 1
		end
	end
	return voxels, broken, #shapes
end

function VecMult(a,b)
	return Vec(a[1]*b[1],a[2]*b[2],a[3]*b[3])
end
