function init()
	joints = FindJoints("engine")
	for j, joint in pairs(joints) do
		SetJointMotor(joint, 20)
	end
	bodies = FindBodies() --gets all bodies under the script because those are all we need (the propellers)
	enginepos = GetLocationTransform(FindLocation("emit")).pos
	enginebreak = 0
	flame = 100
	SetInt("level.engines",2)
	enginebroken = false
	smallsound = {}
	for i=0,5 do
		smallsound[i] = LoadSound("MOD/main/Boeing 737/snd/fail_small/"..i)
	end
	bigsound = {}
	for i=0,3 do
		bigsound[i] = LoadSound("MOD/main/Boeing 737/snd/fail_big/"..i)
	end
end

function tick(dt)
	
	local id = GetIntParam("id")
	red = GetInt("level.engines."..id..".red")
	
	local ptrans = GetPlayerTransform()
	for b, body in pairs(bodies) do
		center = TransformToParentPoint(GetBodyTransform(body),GetBodyCenterOfMass(body))
		voxels, broken, total = GetBodyVoxelCount(body)
		local soundpos = VecLerp(ptrans.pos,center,0.1)
		if voxels == 0 then --completely destroyed = no vfx
			enginebreak = 2
			flame = -150
			if enginebroken == false then
				PlaySound(bigsound[math.random(1,3)],enginepos,100)
				SetInt("level.engines",GetInt("level.engines")-1)
				enginebroken = true
			end
		end
		if broken > 0 and enginebreak == 0 then
			enginebreak = 1
			PlaySound(smallsound[math.random(1,5)],soundpos,5)
		elseif broken > 15 and enginebreak <= 1 then
			enginebreak = 2
			PlaySound(bigsound[math.random(1,3)],soundpos,100)
			if enginebroken == false then
				SetInt("level.engines",GetInt("level.engines")-1)
				enginebroken = true
			end
		end
		if center[2] < 165  and flame > 0 then --when an engine falls off it explodes
			Explosion(center,4)
			enginebreak = 2
			flame = -150
			PlaySound(bigsound[math.random(1,3)],enginepos,100)
			if enginebroken == false then
				SetInt("level.engines",GetInt("level.engines")-1)
				enginebroken = true
			end
		end
	end
	if enginebreak == 2 then --very, very bad things are happening
		if flame > 0 then
			flame = flame - 1
		end
		--first spawn fire
		ParticleReset()
		ParticleColor(1,0.6,0.3,0.1,0.1,0.1,"linear",0,0.5)
		ParticleEmissive(2,0,"linear",0,0.5)
		ParticleRadius(2.75+(flame/400),5,"linear",0.2,1)
		ParticleAlpha(1.0,0.0)
		ParticleCollide(0)
		ParticleGravity(0,-2)
		SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,30),(flame/50)+1.5)
		--next is smoke
		ParticleReset()
		ParticleColor(0.16,0.15,0.15)
		ParticleRadius(3,5)
		ParticleAlpha(0.8,0.0,"linear",0.03,0.9)
		ParticleCollide(0)
		ParticleGravity(0,-2)
		SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,30),(flame/20)+3.5)
	elseif enginebreak == 1 then --bad things are sort of happening
		ParticleReset()
		ParticleColor(0.5,0.4,0.4)
		ParticleRadius(3,5)
		ParticleAlpha(0.3,0.0,"linear",0.03,0.9)
		ParticleCollide(0)
		ParticleGravity(0,-2)
		SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,30),7)
	else --engine is nominal
		ParticleReset()
		ParticleColor(0.9,0.9,0.9)
		ParticleRadius(3,10)
		if red > 0 then
			ParticleAlpha(0.1,0.0,"linear",0.03,0.9)
		else
			ParticleAlpha(0.3,0.0,"linear",0.03,0.9)
		end
		ParticleCollide(0)
		ParticleGravity(0,-2)
		SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,30),20)
	end
	
	if red > 0 and enginebreak < 2 then --funny tomato juice in the engines
		SetInt("level.engines."..id..".red",red-1)
		
		ParticleReset()
		ParticleColor(0.4,0.2,0.2)
		ParticleRadius(3.1,5)
		ParticleAlpha(0.5,0.0,"linear",0.03,0.9)
		ParticleCollide(0)
		ParticleGravity(0,-2)
		SpawnParticle(enginepos,Vec(math.random()-0.5,math.random()-0.5,30),7)
	end
	
	if enginebreak < 2 then
		sceneBodies = FindBodies(nil,true)
		for b=1,#sceneBodies do
			bTrans = GetBodyTransform(sceneBodies[b])
			if VecLength(VecMult(VecSub(bTrans.pos,enginepos),Vec(1,1,0.4))) < 5 then
				ApplyBodyImpulse(sceneBodies[b],bTrans.pos,Vec(0,0,100))
				DrawBodyOutline(sceneBodies[b])
			end
		end
	end
end

function GetBodyVoxelCount(body)
	local broken = 0
	local voxels = 0
	local shapes = GetBodyShapes(body)
	for s, shape in pairs(shapes) do
		voxels = voxels + GetShapeVoxelCount(shape)
		if IsShapeBroken(shape) then
			broken = broken + 1
		end
	end
	return voxels, broken, #shapes
end

function VecMult(a,b)
	return Vec(a[1]*b[1],a[2]*b[2],a[3]*b[3])
end
